@@ -23,7 +23,7 @@ def bs(byte_array):
     return ''.join([chr(c) for c in byte_array])
 
 DEBUG = False
-__version__ = '0.4.4'
+__version__ = '0.5.0'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -396,16 +396,15 @@ class cursor:
     def callproc(self, procname, *params):
         raise NotSupportedError()
 
-    def execute(self, query, params = []):
+    def _execute(self, query, params):
         self.connection._op_prepare_statement(self.stmt_handle, query)
         (h, oid, buf) = self.connection._op_response()
 
         cooked_params = []
         for param in params:        # Convert str/bytes parameter to blob id
-            if type(param) == str: 
+            if type(param) == str:
                 param = self.connection.str_to_bytes(param)
-            elif ((PYTHON_MAJOR_VER==3 and type(param) != bytes) or 
-                            (PYTHON_MAJOR_VER==2 and type(param) != str)):
+            else:
                 cooked_params.append(param)
                 continue
             self.connection._op_create_blob2()
@@ -423,7 +422,7 @@ class cursor:
 
         assert buf[:3] == bs([0x15,0x04,0x00]) # isc_info_sql_stmt_type (4 bytes)
         stmt_type = bytes_to_int(buf[3:7])
-        if stmt_type == 1:  # isc_info_sql_stmt_select
+        if stmt_type == isc_info_sql_stmt_select:
             assert buf[7:9] == bs([0x04,0x07])
             l = bytes_to_int(buf[9:11])
             col_len = bytes_to_int(buf[11:11+l])
@@ -442,21 +441,36 @@ class cursor:
             self.connection._op_execute(self.stmt_handle, cooked_params)
             (h, oid, buf) = self.connection._op_response()
 
-            # Fetch
-            self.rows = []
-            more_data = True
-            while more_data:
-                self.connection._op_fetch(
-                                    self.stmt_handle, calc_blr(self._xsqlda))
-                (rows, more_data) = self.connection._op_fetch_response(
-                                                self.stmt_handle, self._xsqlda)
-                self.rows += rows
+        else:
+            self.connection._op_execute(self.stmt_handle, cooked_params)
+            try:
+                (h, oid, buf) = self.connection._op_response()
+            except OperationalError:
+                e = sys.exc_info()[1]
+                if 335544665 in e.gds_codes:
+                    raise IntegrityError(e.message, e.gds_codes, e.sql_code)
+        return stmt_type
 
-            # Convert BLOB handle to data
-            for i in range(len(self._xsqlda)):
-                x = self._xsqlda[i]
-                if x.sqltype == SQL_TYPE_BLOB:
-                    for r in self.rows:
+    def execute(self, query, params = []):
+        self._stmt_type = self._execute(query, params)
+        if self._stmt_type == isc_info_sql_stmt_select:
+            self._fetch_records = self._fetch_generator()
+
+    def executemany(self, query, seq_of_params):
+        for params in seq_of_params:
+            self.execute(query, params)
+
+    def _fetch_generator(self):
+        more_data = True
+        while more_data:
+            self.connection._op_fetch(self.stmt_handle, calc_blr(self._xsqlda))
+            (rows, more_data) = self.connection._op_fetch_response(
+                                            self.stmt_handle, self._xsqlda)
+            for r in rows:
+                # Convert BLOB handle to data    
+                for i in range(len(self._xsqlda)):    
+                    x = self._xsqlda[i]    
+                    if x.sqltype == SQL_TYPE_BLOB:    
                         if not r[i]:
                             continue
                         self.connection._op_open_blob(r[i])
@@ -473,44 +487,31 @@ class cursor:
                         self.connection._op_close_blob(h)
                         (h, oid, buf) = self.connection._op_response()
                         r[i] = v
-            self.cur_row = 0
-            # recreate stmt_handle
-            self.connection._op_free_statement(self.stmt_handle, 2) # DSQL_drop
-            (h, oid, buf) = self.connection._op_response()
-            self.connection._op_allocate_statement()
-            (h, oid, buf) = self.connection._op_response()
-            self.stmt_handle = h
-        else:
-            self.connection._op_execute(self.stmt_handle, params)
-            try:
-                (h, oid, buf) = self.connection._op_response()
-            except OperationalError:
-                e = sys.exc_info()[1]
-                if 335544665 in e.gds_codes:
-                    raise IntegrityError(e.message, e.gds_codes, e.sql_code)
-            self.rows = None
+                yield r
 
-    def executemany(self, query, seq_of_params):
-        for params in seq_of_params:
-            self.execute(query, params)
+        # recreate stmt_handle
+        self.connection._op_free_statement(self.stmt_handle, 2) # DSQL_drop
+        (h, oid, buf) = self.connection._op_response()
+        self.connection._op_allocate_statement()
+        (h, oid, buf) = self.connection._op_response()
+        self.stmt_handle = h
+
+        raise StopIteration()
             
     def fetchone(self):
-        if self.cur_row < len(self.rows):
-            r = self.rows[self.cur_row]
-            self.cur_row += 1
-            return r
-        else:
+        try:
+            if PYTHON_MAJOR_VER==3:
+                return next(self._fetch_records)
+            else:
+                return self._fetch_records.next()
+        except StopIteration:
             return None
 
     def fetchall(self):
-        rows = []
-        r = self.fetchone()
-        while r:
-            rows.append(r)
-            r = self.fetchone()
-        return rows
+        return list(self._fetch_records)
 
     def fetchmany(self, size=None):
+        rows = []
         if not size:
             size = self.arraysize
         r = self.fetchone()
@@ -522,12 +523,6 @@ class cursor:
             r = self.fetchone()
         return rows
 
-
-        rows = self.rows[self.cur_row:self.cur_row+size]
-        self.cur_row += size
-        if self.cur_row > len(self.rows):
-            self.cur_row = len(self.rows)
-
     def close(self):
         if not hasattr(self, "stmt_handle"):
             return
@@ -552,10 +547,7 @@ class cursor:
                         x.sqlscale, True if x.null_ok else False))
             return r
         elif attrname == 'rowcount':
-            if self.rows:
-                return len(self.rows)
-            else:
-                return -1
+            return -1
         raise AttributeError
 
     def __del__(self):
@@ -665,15 +657,13 @@ class BaseConnect:
         values = bs([])
         for p in params:
             t = type(p)
-            if t == str:
-                v = self.str_to_bytes(p)
+            if ((PYTHON_MAJOR_VER == 2 and t == str) or
+                (PYTHON_MAJOR_VER == 3 and t == bytes)):
+                v = p
                 nbytes = len(v)
                 pad_length = ((4-nbytes) & 3)
                 v += bs([0]) * pad_length
                 blr += bs([14, nbytes & 255, nbytes >> 8])
-            elif PYTHON_MAJOR_VER == 3 and t == bytes:
-                v = p
-                blr += bytes([9, 0])
             elif t == int:
                 v = bint_to_bytes(p, 4)
                 blr += bytes([7, 0])
